/* decompress 16-bit floating point photocurrents (4-bit exponent, 12-bit mantissa) that have been
 * compressed using Rice compression.
 *
 * Usage: decomp16b infile outfile 
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

		     
int main(argc,argv)
    int argc;
    char *argv[];
{
    int infile, outfile;    /* file descriptors */
    unsigned char *cbuf;    /* buffer containing compressed block */
    unsigned short npix;    /* num of pixels in each block to be compressed */
    unsigned short nblocks; /* num of blocks per image */
    unsigned short *samples;
    int *minimums;
    int ii,jj;

    void decompress_flags(unsigned char *cflags,unsigned char *flags,unsigned char cfsize,unsigned short npix);
    char readnextbit(unsigned char *cflags,int bit,int byte);

    if (argc!=3) {
       fprintf(stderr,"Usage: %s infile outfile \n",argv[0]);
       exit(1);
    }

    if ((infile=open(argv[1],O_RDONLY)) == -1) {
       fprintf(stderr,"Error: %s could not open infile\n",argv[0]);
       exit(1);
    }

    if ((outfile=open(argv[2],O_CREAT | O_RDWR | O_TRUNC,S_IRUSR | S_IWUSR)) == -1) {
       fprintf(stderr,"Error: %s could not open outfile\n",argv[0]);
       exit(1);
    }

    /* read file header - npix and nblocks */
    
    if (read(infile,&nblocks,sizeof(unsigned short)) != sizeof(unsigned short)) {
	fprintf(stderr,"%s: Error: could not read nblocks.\n",argv[0]);
	exit(-1);
    }
    if (read(infile,&npix,sizeof(unsigned short)) != sizeof(unsigned short)) {
	fprintf(stderr,"%s: Error: could not read npix.\n",argv[0]);
	exit(-1);
    }


    fprintf(stdout,"%u %u\n",nblocks,npix);

    samples= (unsigned short *) malloc(npix * nblocks * sizeof(unsigned short));
    minimums= (int *) malloc(nblocks * sizeof(int));


    for (ii=0;ii<nblocks;ii++) {
	fprintf(stdout,"%5d\n",ii);
	unsigned char hdr[6];
	if (read(infile,hdr,6) != 6) {
	    fprintf(stderr,"Error: %s hdr read error %d\n",argv[0],ii);
	    exit(1);
	}
	unsigned char nbits;
	short min;
	unsigned char cfsize;
        unsigned char flags[npix];
	nbits = hdr[1] & 0xF;

	min = ((hdr[0] & 0x7F) <<4) + ((hdr[1]>>4) & 0xF);
	if ((hdr[0] & 0x80)==0x80) {
	   min = -min;
	}
	minimums[ii] = min;

	cfsize = hdr[2];
	fprintf(stdout,"%04x %04x\n",hdr[0],hdr[1]);
	fprintf(stdout,"%2u %3u   %5d\n",nbits,cfsize,min);

	unsigned char cflags[cfsize];
	if (read(infile,cflags,cfsize) != cfsize) {
	    fprintf(stderr,"Error: %s cflags read error %d\n",argv[0],ii);
	    exit(1);
	}
/*
	fprintf(stdout,"%0x %0x %0x %0x %0x %0x %0x %0x %0x %0x %0x\n",
	cflags[0],cflags[1],cflags[2],cflags[3],cflags[4],cflags[5],cflags[6],cflags[7],cflags[8],cflags[9],cflags[10]);
*/	

	decompress_flags(cflags,flags,cfsize,npix);

	int bitstoread = 0;
	for (jj=0;jj<npix;jj++) {
	    if (flags[jj] == 0) {
		bitstoread += nbits;
	    } else {
		bitstoread += 16;
	    }
	}
	int bytestoread;
	bytestoread = bitstoread / 8;
	if (bitstoread %8 != 0) {
	    bytestoread++;
	}
        unsigned char cdata[bytestoread];
        if (read(infile,cdata,bytestoread) != bytestoread) {
            fprintf(stderr,"Error: %s cdata read error %d\n",argv[0],ii);
            exit(1);
	}
/*	fprintf(stdout,"%0x %0x %0x %0x %0x %0x\n",cdata[0],cdata[1],cdata[2],cdata[3],cdata[4],cdata[5]); */
	int bit = 0;
	int byte = 0;
	for (jj=0;jj<npix;jj++) {
	    if (flags[jj] == 0) {
		bitstoread = nbits;
		
	    } else {
		bitstoread = 16;
	    }
	    int kk;
	    unsigned short sample = 0;
	    char nextbit;
	    for (kk=0;kk<bitstoread;kk++) {
		nextbit = readnextbit(cdata,bit++,byte);
		sample |= (nextbit << kk);
		if (bit==8) {
		    byte++;
		    bit=0;
		}
	    }
	    samples[(ii*npix) + jj] = sample; 
	    fprintf(stdout,"%04x ",sample);
	    if (jj%8==7)
		fprintf(stdout,"\n");
	}
	fprintf(stdout,"\n");
    }
    write(outfile,minimums,nblocks * sizeof(int));
    write(outfile,samples,npix * nblocks * sizeof(unsigned short));

    close(infile);
    close(outfile);
}

unsigned short codewords[8] = {0,4,5,28,6,29,30,31}; /* 3-bit encoding codewords from Rice paper, Table 2-1 */
int codebits[8] = {1,3,3,5,3,5,5,5};        /* number of bits for each codeword */
	

void decompress_flags(unsigned char *cflags,unsigned char *flags,unsigned char cfsize,unsigned short npix) {
    int ii,jj;
    int bit = 0;
    int byte = 0;
    char nextbit;
    char codeword=0;
    int cidx=0;
    int fidx=0;
    int threebits;
    int done = 0;

    char readnextbit(unsigned char *cflags,int bit,int byte);
    int codeidx(char codeword);

    for (ii=0;ii<cfsize*8;ii++) {
	nextbit=readnextbit(cflags,bit++,byte);
/*	codeword |= (nextbit << cidx); */
	codeword = (codeword << 1) + nextbit; 
	cidx++;
	threebits=codeidx(codeword);
	if (threebits >= 0) {
	    for(jj=0;jj<3;jj++) {
		flags[fidx++] = (threebits >> jj) & 0x1;
		if (fidx==npix) {
		    done = 1;
		    break;
		}
	    }
	    cidx=0;
	    codeword = 0;
	} else {
	    if (cidx>=5) {
		fprintf(stderr,"Flags Decode error %d %d %d\n",codeword,nextbit,cidx);
		exit(1);
	    }
	}

	fprintf(stdout,"%1d ",nextbit);
	fflush(stdout);
	if (ii%8==7)
	    fprintf(stdout,"\n");
	if (bit==8) {
	    byte++;
	    bit=0;
	}
	if (done) break;
    }
    fprintf(stdout,"\n");
	
}

char readnextbit(unsigned char *buf,int bit,int byte) {
    return((buf[byte] >> bit) & 0x1);
}

int codeidx(char codeword) {
    int ii;
    int idx = -1;
    for (ii=0;ii<8;ii++){
	if (codeword==codewords[ii]) {
	    idx = ii;
	    break;
	}
    }
    return(idx);
}
