/* 32-bit -> 16-bit compression */

/*
This algorithm is a modified biased exponent, hidden one algorithm
with a 12 bit mantissa and a 4 bit exponent. Numbers up to 2^12 are
uncompressed, and numbers up to 2^13 decompress with no error.

Numbers up to 4095 * 2^15 (134,184,960) can be compressed.
*/

unsigned int num_bits=16, num_mantissa_bits=12, num_exponent_bits=4;
unsigned int output_mask=0xffff; /* 0xffff 16 bit, 0x0fff 12 bit */


#include <stdio.h>
#include <math.h>
#include <stdlib.h>


unsigned int pack_rate(ratein)
int ratein;
{
    unsigned int rateout, power=0;
    unsigned int mask;
    mask = (0xffffffff<<(num_mantissa_bits+1));
    while (ratein&mask)
    {
	power += (0x0001<<num_mantissa_bits);
	ratein>>=1;
	}
    rateout=ratein;
    if (power)
    {
	rateout = power + (0x0001<<num_mantissa_bits)
	| (rateout & (output_mask>>num_exponent_bits));
}
    rateout = (rateout & output_mask);
    return rateout;
}

/* Unpacking (not required in flight code) */
/* switched from int to unsigned int -- AWL 011107 */
unsigned int int_rate(packed) /* Unpack to int */
unsigned int packed; /* was just int. AWL 030908 */
{
    unsigned int power; /* was just int. AWL 030908 */
    unsigned int out; /* was just int. AWL 030908 */
    power = packed>>num_mantissa_bits;
    if (power>1)
    {
	out = ( (packed & (output_mask>>num_exponent_bits))
	       | (0x0001<<num_mantissa_bits) );
	out = out<<(power-1);
    }
    else
    out = packed;
    return out;
}


double dbl_rate(packed) /* Unpack to double */
unsigned packed;
{
    int power;
    double out;
    power = packed>>num_mantissa_bits;
    if (power>1)
    {
	out = ( (packed & (output_mask>>num_exponent_bits))
	       | (0x0001<<num_mantissa_bits) );
	out = out * pow(2.,(double)(power-1));
    }
    else
    out = packed;
    return out;
}
