#!/bin/sh

#  Read photocurrents from a given simulated image
./imread ./sim_exposure_095830_array_2.fits\[0\] > foo

# Pack the photocurrents into 16-bit format (4 bits exponent, 12 bits mantissa) and write
# results to a binary file.
# Use 256 pixels per block, and a bin-width of 0.02 e-
./pack16b_v02 256 0.02 foo.pak < foo > foo.log

# Compress the packed photocurrents using Rice compression scheme
./comp16b_v02 foo.pak foo.cmp 256 1 > foo.stat




# Decompress the compressed photocurrents.
./decomp16b_v02 foo.cmp foo.decompressed >& foo.decomp.log

# Check that the decompressed data are the same as the original packed data.
# i.e. did the Rice compression work as expected 
diff foo.decompressed foo.pak

# unpack the decompressed data and undo the 0.02 e- binning
./unpack16b_v02 foo.decompressed 256 | awk '{print $5 * 0.02}' > foo.unpacked

# reformat the output of imread to have just one photocurrent per line
awk '{for(ii=1;ii<=NF;ii++)print $ii}' foo > foo.1perline

# Compare the imread output with the unpacked/decompressed results.
# They will not be exactly the same, since the application of the 0.02e- bin-width is "lossy".
echo original	decompressed	diff > foo.compared
paste -d" " foo.1perline foo.unpacked | awk '{printf("%6.3f\t%6.3f\t%6.3f\n",$1,$2,$1-$2)}' >> foo.compared
