/* read 16-bit floating point photocurrents (4-bit exponent, 12-bit mantissa) 
/* Data are read in blocks from a binary input file containing 2048x2048 16-bit photocurrents.
 * ascii text data are written to stdout
 *
 * Usage: read16b_v02 infile npix_per_block
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "packsubs.c"

#define IMSIZE 2048 * 2048
		     
int main(argc,argv)
    int argc;
    char *argv[];
{
    int infile;		    /* file descriptors */
    int *minimums;	    /* buffer containing a block of minimums, one for each block of 16-bit photocurrents */
    unsigned short *inbuf;  /* buffer containing a block of 16-bit photocurrents	*/
    int npix;		    /* num of pixels in each block to be compressed */
    int nblocks;	    /* num of blocks per image */
    int ii,jj;
    unsigned int tmp,unpacked;


    unsigned int int_rate();  /* Unpack to int */

    if (argc!=3) {
       fprintf(stderr,"Usage: %s infile npix_per_block\n",argv[0]);
       exit(1);
    }

    if ((infile=open(argv[1],O_RDONLY)) == -1) {
       fprintf(stderr,"Error: %s could not open infile\n",argv[0]);
       exit(1);
    }

    npix = atoi(argv[2]);
    if (npix<0) {
        fprintf(stderr,"Error: %s npix_per_block %d is negative\n",argv[0],npix);
        exit(1);
    }
    if (npix>IMSIZE) {
        fprintf(stderr,"Error: %s npix_per_block %d is greater than image size %d\n",argv[0],npix,IMSIZE);
        exit(1);
    }
    if (npix>65535) {
        fprintf(stderr,"Error: %s npix_per_block %d is greater than 65535\n",argv[0],npix);
        exit(1);
    }
    if (IMSIZE % npix) {
        fprintf(stderr,"Error: %s image size %d must be divisible by npix_per_block %d\n",argv[0],IMSIZE,npix);
        exit(1);
    }

    nblocks = IMSIZE / npix;


    minimums= (int *) malloc(nblocks * sizeof(int));
    inbuf= (unsigned short *) malloc(npix * sizeof(unsigned short));

    if (read(infile,minimums,nblocks * sizeof(int)) != (nblocks * sizeof(int))) {
        fprintf(stderr,"Error: %s read minimums error %d\n",argv[0],ii);
        exit(1);
    }

    for (ii=0;ii<nblocks;ii++) {
	if (read(infile,inbuf,npix * sizeof(unsigned short)) != (npix * sizeof(unsigned short))) {
	    fprintf(stderr,"Error: %s read error %d\n",argv[0],ii);
	    exit(1);
	}

	for (jj=0;jj<npix;jj++) {
	    tmp = (unsigned int)inbuf[jj];
	    unpacked = int_rate(tmp);
	    fprintf(stdout,"%5d     %04x  %8u %8d %8d\n",ii,inbuf[jj],unpacked,  minimums[ii], (int)unpacked + minimums[ii]);
	}

    }

    close(infile);
}

